/*
 * IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING. By
 * downloading, copying, installing or using the software you agree to
 * this license. If you do not agree to this license, do not download,
 * install, copy or use the software.
 * 
 * Intel Open Source License 
 * 
 * Copyright (c) 2005 Intel Corporation. All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of the Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *  
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INTEL OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _OASYS_GETOPT_H_
#define _OASYS_GETOPT_H_

#include <string>
#include <vector>
#include "Options.h"

namespace oasys {

/*
 * Wrapper class for getopt calls.
 */
class Getopt {
public:
    /**
     * Register a new option binding.
     */
    static void addopt(Opt* opt);

    /**
     * Parse argv, processing all registered getopt. Returns the
     * index of the first non-option argument in argv
     * 
     * @param progname   	the name of the executable
     * @param argc	   	count of command line args
     * @param argv   		command line arg values
     * @param extra_opts 	additional usage string
     */
    static int getopt(const char* progname, int argc, char* const argv[],
                      const char* extra_usage = "");

    /**
     * Prints a nicely formatted usage string to stderr.
     *
     * @param progname   	the name of the executable
     * @param extra_opts 	additional usage string
     */
    static void usage(const char* progname,
                      const char* extra_usage = "");
    
protected:
    typedef std::vector<Opt*> OptList;
    
    static Opt* opts_[];	// indexed by option character
    static OptList allopts_;	// list of all getopt
};

} // namespace oasys

#endif /* _OASYS_GETOPT_H_ */
