/*
 * IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING. By
 * downloading, copying, installing or using the software you agree to
 * this license. If you do not agree to this license, do not download,
 * install, copy or use the software.
 * 
 * Intel Open Source License 
 * 
 * Copyright (c) 2005 Intel Corporation. All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of the Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *  
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INTEL OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "SchemeTable.h"
#include "DTNScheme.h"
#include "EthernetScheme.h"
#include "StringScheme.h"
#include "WildcardScheme.h"
#include "TCAScheme.h"

namespace dtn {

template <>
SchemeTable* oasys::Singleton<SchemeTable>::instance_ = 0;

/**
 * Constructor -- instantiates and registers all known schemes. Called
 * from the singleton instance() method the first time the table is
 * accessed.
 */
SchemeTable::SchemeTable()
{
    table_["dtn"]	= DTNScheme::instance();
    table_["str"]	= StringScheme::instance();
    table_["*"]		= WildcardScheme::instance();
#ifdef __linux__
    table_["eth"]	= EthernetScheme::instance();
#endif
    table_["tca"]   = TCAScheme::instance();
}

/**
 * Find the appropriate Scheme instance based on the URI
 * scheme of the endpoint id scheme.
 *
 * @return the instance if it exists or NULL if there's no match
 */ 
Scheme*
SchemeTable::lookup(const std::string& scheme_str)
{
    SchemeMap::iterator iter = table_.find(scheme_str);
    if (iter == table_.end()) {
        return NULL;
    }
    
    return (*iter).second;
}

}
