/*
 * IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING. By
 * downloading, copying, installing or using the software you agree to
 * this license. If you do not agree to this license, do not download,
 * install, copy or use the software.
 * 
 * Intel Open Source License 
 * 
 * Copyright (c) 2004 Intel Corporation. All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 *   Neither the name of the Intel Corporation nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *  
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE INTEL OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _TCL_REGISTRATION_H_
#define _TCL_REGISTRATION_H_

#include <oasys/debug/Log.h>
#include <oasys/tclcmd/TclCommand.h>
#include <oasys/thread/Thread.h>

#include "Registration.h"
#include "bundling/Bundle.h"

namespace dtn {

class BlockingBundleList;


/**
 * A simple utility class used mostly for testing registrations.
 *
 * When created, this sets up a new registration within the daemon,
 * and for any bundles that arrive, outputs logs of the bundle header
 * fields as well as the payload data (if ascii). The implementation
 * is structured as a thread that blocks (forever) waiting for bundles
 * to arrive on the registration's bundle list, then tcl the
 * bundles and looping again.
 */
class TclRegistration : public Registration {
public:
    TclRegistration(const EndpointIDPattern& endpoint,
                    Tcl_Interp* interp);
    int exec(int argc, const char** argv, Tcl_Interp* interp);

    /**
     * Return in the tcl result a Tcl_Channel to wrap the BundleList's
     * notifier pipe.
     */
    int get_list_channel(Tcl_Interp* interp);

    /**
     * Assuming the list channel has been notified, pops a bundle off
     * the list and then returns in the tcl result a list of the
     * relevant metadata and the payload data.
     */
    int get_bundle_data(Tcl_Interp* interp);

    /**
     * Parse the given bundle's internals into a new tcl list object
     * (or an error if parsing fails).
     */
    static int parse_bundle_data(Tcl_Interp* interp,
                                 const BundleRef& b,
                                 Tcl_Obj** result);

    /// virtual from Registration
    void deliver_bundle(Bundle* bundle);

protected:
    BlockingBundleList* bundle_list_;
    Tcl_Channel notifier_channel_;
};

} // namespace dtn

#endif /* _TCL_REGISTRATION_H_ */
